#!/usr/bin/perl

# Tabsize: 3


use strict;
use warnings;


=head1 NAME

check_dns.pl -- Testet, ob fremde Nameserver andere (fremde) Hosts korrekt aufloesen

=head1 SYNOPSIS

check_dns [--check <FQDN>] [--reference <nameserver>] [--good] [<File mit Domains>]

Optionen:

  --check        Full Qualified Domain Name des zu testenden Systems
                 kann mehrfach angegeben werden
                 Default: www.nazi-lauck-nsdapao.com, www.stormfront.org,
                 www.rotten.com

  --reference    Referenz-Nameserver, der korrekte Ergebnisse liefert
                 Default: localhost

  --good         FQDN eines Rechners, der auf jeden Fall korrekt aufgeloest wird
                 Als Gegentest, ob der fremde Nameserver ueberhaupt irgendwas
                 aufloest
                 Default: proxy.odem.org

  --verbose      Sei geschwaetzig (default)
  --noverbose    Sei NICHT geschwaetzig!
  
  <file mit Domains> Datei, in der Domains stehen, deren Nameserver getestet
                     werden sollen


=head1


=head1 DESCRIPTION

Testet, ob die fuer eine Reihe von Domains zustaendigen Nameserver korrekte
Ergebnisse fuer fremde Hosts zurueckliefern.
Damit laesst sich z.B. testen, ob fremde Nameserver gefaelschte Nameserver-Eintraege
liefern.
Sprich: es laesst sich nutzen, um festzustellen, ob fremde Nameserver die
Buessowsche "Sperr"-Methode anwenden


Das Programm geht davon aus, dass der lokale Computer auf die Original-Eintraege
zugreift


=head1 EXAMPLES

./check_dns.pl

Anschliessend Zeilenweise Domains eingeben, deren Nameserver getestet werden sollen.
Ende mit ^D


./check_dns.pl domainlist.txt

'domainlist.txt' ist eine Textdatei, in der zeilenweise einzelne Domains stehen.
Die fuer die entsprechende Zone zustaendigen Nameserver werden dann nach den
Default-Check-Rechnern abgefragt


./check_dns.pl --check www.apel-au-peuples.org --check www.hutzelwicht.de \
               --reference good-ns.host.tld --good www.test.de domainlist.txt


Testet bei den Nameservern aus domainlist.txt, ob www.apel-au-peuples.org und
www.hutzelwicht.de korrekt aufgeloest werden. good-ns.host.tld liefert die
korrekten Daten fuer die Domains. www.test.de wird herangezogen, um zu schauen,
ob ueberhaupt externe Domains remote abgefragt werden koennen.


=head1 Hinweise

Die durch dieses Programm vorgenommenen Tests gehen von der Annahme aus,
dass der fuer eine Zone zustaendige Nameserver auch der Nameserver fuer Clients
innerhalb der Zone (Domain) sind. Dies muss nicht zwangsweise der Fall sein!

Einige Nameserver erlauben auch nicht von externen Hosts, fremde Domains 
abzufragen.


WICHTIG:
Ein DNS-Test kann *nie* genau sagen, ob eine bestimmte Webseite erreicht 
werden kann. Es gibt diverse weitere Moeglichkeiten der "Sperrung" oder
von Kommunikationsfehlern, z.B. via Firewall-Regeln, Routin, Zwangsproxies 
usw.


=head1 BUGS

puh, bestimmt zu viele ...

Die End-Ausgabe koennte noch etwas komfortabler sein.


=head1 COPYRIGHT

(c) 2003 Alvar C.H. Freude, http://alvar.a-blast.org/

Dieses Programm steht unter der GNU Public Licence (GPL)


=cut


use Getopt::Long;
use Pod::Usage;

use Net::DNS;

use Data::Dumper;


#
# Besetze Default-Werte
#

my $man = 0;
my $help = 0;

my $VERBOSE = 1;           # Wir sind geschwaetzig!


my @default_check = qw(www.nazi-lauck-nsdapao.com
               www.stormfront.org
               www.rotten.com);

my @check;

my $reference = "localhost";

my $good = "proxy.odem.org";


#
# Hole Parameter
#

GetOptions
   (
   'help|?'    => \$help,
	man         => \$man,
   'check=s'       => \@check,
   'reference=s'   => \$reference,
   'good=s'        => \$good,
   'verbose!'  => \$VERBOSE,
   ) or pod2usage(2);

pod2usage(1) if $help;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;


@check = @default_check unless @check;

#
# Hole die Referenz-Werte 
#

print "Hole Referenz-Werte ...\n" if $VERBOSE;

my $ok = get_reference($reference, @check, $good);

unless ($ok)
   {
   print "Der Referenz-Nameserver konnte nicht alles korrekt aufloesen!\n";
   print "Bitte mit einem anderen Server versuchen\n";
   print "($reference)\n";
   exit;
   }

if ($VERBOSE)
   {
   print "\nReferenz-Werte aufloesen OK:\n";
   print Dumper($ok);
   }


#
# Schleife ueber alle zu testenden Domains bzw. deren Nameserver
#

my %status = ();

print "\nNun der Haupt-Test ...\n" if $VERBOSE;

while (my $domain = <>)
   {
   chomp $domain;             # Newline von der Eingabezeile (Domain) abschneiden
                              # den eigentlichen Check fuer die Nameserver der Domain machen
   $status{$domain} = do_check($domain, $ok, $good, @check);

                              # Geschwaetz ausgeben ...
   print "\nStatus von $domain:\n",
      $status{$domain} ?
         Dumper($status{$domain}) : "Von aussen nicht erlaubt",
      "\n" if $VERBOSE;
   }

print "Ergebnis:\n=========\n\n";
print Dumper(\%status);



# Ciao!

exit;


=head1 Interne Dokumentation

=== Interne Dokumentation =================================================

=cut


# === Basis-Funktionen ====================================================


=item get_reference($reference, @check, [$good])

Testet alle zu testenden Namen mal bei einem vertrauenswuerdigen Nameserver durch

Parameter:

   $reference   Referenz-Nameserver, also einer, der Originaldaten liefert
   @check       Array mit den zu testenden FQDNs
   ($good       Test-FQDN, der korrekt aufgeloest werden sollte)
                geht in @check auf


Rueckgabe:

  Referenz auf hash mit Key-Value-Paaren: domainname => ip

  Bei Fehlern: undef


=cut


sub get_reference
   {
   my ($reference_ns, @check) = @_;
   my %reference = ();

   foreach my $fqdn (@check)        # Fuer jeden Namen die IP holen
      {                             # Wenn was schiefgeht: abbrechen
      $reference{$fqdn} = get_ip_from_name($fqdn, $reference_ns) || return undef;
      }

   return \%reference;              # eine Referenz auf den Hash mit den als 
                                    # korrekt getesteten Daten zurueckgeben
   }



=item do_check($domain, $oklist, $good, @check)

Testet, ob der Nameserver einer Domain eine Reihe von fremden Hosts korrekt aufloest

Parameter:

  $domain   Die Domain, in der die Nameserver getestet werden sollen
  $oklist   Hash, mit korrekten Werten der einzelnen zu testenden Domains
  $good     Ein Host, der immer korrekt aufgeloest werden sollte
  @check    Liste mit zu testenden Hosts

Rueckgabe:

undef: auch der korrekt aufzuloesende Host konnte nicht aufgeloest werden.
i.d.R. werden also fremde Hosts von diesem Domainserver von externen Nutzern nicht
aufgeloest oder ein anderer aehnlich schwerwiegender Fehler trat ein.

Referenz Hash mit je einem Key fuer jeden NS der zu testenden Domain; value ist
eine Referenz auf einen Hash mit je einem Eintrag zu jedem Host
Folgende Moeglichkeiten:

  "OK!"            Alles korrekt aufgeloest
  "WRONG: <ip>"    host konnte aufgeloest werden, aber falsche IP ...
  "ERROR: <...>"   Sonstige Fehlermeldung
  "DENIED: <...>"  Habe gar nichts zurueckgekriegt


=cut

sub do_check
   {
   my ($domain, $oklist, $good, @check) = @_;
   my %status = ();

   my $res     = Net::DNS::Resolver->new;    # Resolver basteln
   my $query   = $res->query($domain, "NS"); # NS fuer die Domain holen
  
   if ($query)                               # Ergebnis?
      {
      foreach my $rr ($query->answer)        # Ja: alle abarbeiten
         {
         next unless $rr->type eq "NS";      # Nur NS Records beachten.
         my $ns = $rr->nsdname;
         print "NS: ", $ns, "\n" if $VERBOSE;# Geschwaetz ...
                                             # Hole die "gute" IP
         my ($addr, $error) = get_ip_from_name($good, $ns);
         
         if ($addr)                          # Es gab was zureuck ... 
            {                                # setze den Status entsprechend
            $status{$ns}->{$good} = ($addr eq $oklist->{$good}) ? "OK!" : "WRONG: $addr";
            }
         else                                # Noe, der Nameserver mag uns nicht
            {
            $status{$ns}->{$good} = "DENIED: $error";
            }


            foreach my $host (@check)        # Nun das gleiche fuer jeden zu testenden Host
               {
               my ($addr, $error) = get_ip_from_name($host, $ns);
               
               if ($addr)                    # Es gab was zureuck ... 
                  {                          # setze den Status entsprechend
                  $status{$ns}->{$host} = ($addr eq $oklist->{$host}) ? "OK!" : "WRONG: $addr";
                  }
               else                          # Noe, der Nameserver mag uns nicht
                  {
                  $status{$ns}->{$host} = "ERROR: $error";
                  }
               print "  ", $status{$ns}->{$host}, "\n" if $VERBOSE;
               }



         print "  ", $status{$ns}->{$good}, "\n" if $VERBOSE;
         }
      }
   else
      {
      return undef;                          # Fehler: keine NS gefunden etc.
      }

   return \%status;

   }


# === DNS-Funktionen ====================================================

=item get_ip_from_name($fqdn, $nameserver)

Hole die IP-Adresse(n), die zu einem Namen gehoeren.

Parameter:

  $fqdn        Name, nach dem geschaut werden soll
  $nameserver  Nameserver, bei dem nachgeschaut werden soll


Rueckgabe:

Die IP-Adressen, die zum Namen gehoeren als String. Kommagetrennt

Im Fehlerfalle undef UND den Error-String (2. Parameter)


=cut


sub get_ip_from_name
   {
   my ($fqdn, $nameserver) = @_;
   my @addrs = ();

   my $res = Net::DNS::Resolver->new;  # Neues Resolver-Objekt bauen
   $res->nameservers($nameserver);     # Nameserver setzen

   my $query = $res->search($fqdn);    # Anfrage starten
  
   if ($query)                         # Anfrage kam durch?    
      {                                # ja: alle Ergebnisse zusammenkleben
      foreach my $rr ($query->answer)
         {
         next unless $rr->type eq "A"; # Nur A-Records interessieren uns
         push @addrs, $rr->address;    # Diese dann erstmal im Array ablegen
         }
      }
   else
      {
      print "FEHLER -- NS: $nameserver; Query: $fqdn\n", $res->errorstring, "\n";
      return undef, $res->errorstring;
      }
   
   my $addrs = join(",", @addrs);
   print "  $fqdn, $nameserver: $addrs\n" if $VERBOSE;
   return $addrs;
   
   }









